
#include <Servo.h>
#include "pitches.h"

#define AI_Taster A0  //Pin A0  
#define DO_POWER 2
#define DO_LED3 3     //PWM on pin 3, 5, 6, 9, 10, and 11. 
#define DO_LED5 5
#define DO_LED7 6
#define DO_SERVO  9
#define DO_SOUND  10
#define DO_SOUND_GND 11

#define POS_UPPER 87
#define POS_MIDDLE 60
#define POS_LOWER 3

unsigned long t;
int timer3s=0;
int timeIndex=0;
int timeTableCount=4;
int timeTable[] = {180, 300, 420, 600};
long int ticCount=0;
float pos = POS_UPPER;
byte triangle=0;
byte dir=5;
int frq=3300;
Servo myservo;  // create servo object to control a servo 


int melody[] = {NOTE_GS4, NOTE_E4, NOTE_FS4, NOTE_B3, NOTE_B3, NOTE_FS4, NOTE_GS4, NOTE_E4};
int noteDurations[] = {2, 2,2,1 , 2,2,2,1};

//--------------------------------------------------------------------------
bool taste(void) {
//--------------------------------------------------------------------------
    do {
      int x = analogRead(AI_Taster);
      //Serial.println(x); //bei voller Batterie: 0 oder 721
        if (x > 150) return true;
        if (x <  20) return false;
    } while (1);    
}

//--------------------------------------------------------------------------
bool entprellteTaste(void) {
//--------------------------------------------------------------------------
static int on=0;
static int off=0;
static bool Q=0;
    if (taste() ){
       if (on < 4) on++; else Q=true;
       off =0;
    } else {
       if (off < 4) off++; else Q=false; 
        on=0;
    }
    return Q;
}

//--------------------------------------------------------------------------
bool keypress(void) {
//--------------------------------------------------------------------------
static bool old=false;
bool key = entprellteTaste();
bool Q = false; 
    if (key && (old != key)) Q = true;
    old = key;
    return Q;
}

//--------------------------------------------------------------------------
void showLeds(int n, byte brightness) { //n is timeIndex
//--------------------------------------------------------------------------
    analogWrite(DO_LED3, 0); 
    analogWrite(DO_LED5, 0); 
    analogWrite(DO_LED7, 0); 
    switch(n) {
        case 0:  analogWrite(DO_LED3, brightness); break;
        case 1:  analogWrite(DO_LED5, brightness); break;
        case 2:  analogWrite(DO_LED7, brightness); break;
        case 3:  analogWrite(DO_LED3, brightness); 
                 analogWrite(DO_LED7, brightness); break;
    }
}

//--------------------------------------------------------------------------
void westminster_Sound(){
//--------------------------------------------------------------------------
    for (int thisNote = 0; thisNote < 8; thisNote++) {
    int noteDuration = 1000 / noteDurations[thisNote];
    tone(DO_SOUND, melody[thisNote], noteDuration);
    int pauseBetweenNotes = noteDuration * 1.30;
    delay(pauseBetweenNotes);
    noTone(DO_SOUND);
  }
}

//--------------------------------------------------------------------------
void lift(float pos){
  myservo.write(180-(int)pos);
}

//--------------------------------------------------------------------------
void setup() {
//--------------------------------------------------------------------------
  pinMode(DO_POWER, OUTPUT); 
  digitalWrite(DO_POWER, HIGH); 

  pinMode(DO_LED3, OUTPUT); 
  pinMode(DO_LED5, OUTPUT); 
  pinMode(DO_LED7, OUTPUT); 
  pinMode(DO_SERVO, OUTPUT); 
  pinMode(DO_SOUND, OUTPUT);  
  
  pinMode(DO_SOUND, OUTPUT);        // Lautsprecher Ausgang Signal
  pinMode(DO_SOUND_GND, OUTPUT);    // Lautsprecher Ausgang Masse
  digitalWrite(DO_SOUND_GND, LOW); 
  Serial.begin(9600);
  myservo.attach(DO_SERVO);  // attaches the servo on pin 9 to the servo object
 
  timer3s = 0;
  lift(POS_UPPER);
  
   digitalWrite(DO_LED3, HIGH); 
   analogWrite(DO_LED5, 250); 
   analogWrite(DO_LED7, 250); 
   tone(DO_SOUND,4000,50);

  //read and discard first analog values
  analogRead(AI_Taster);
  analogRead(AI_Taster);
  analogRead(AI_Taster);
  while (entprellteTaste() || taste()){
    delay(20);
  }
  timeIndex=0;
  triangle=80;
  t = millis();
}


//--------------------------------------------------------------------------
void loop() {
//--------------------------------------------------------------------------
static byte state=0;
bool key;  
  if ((millis() - t) >= 20) { //20 ms cycle time
    t+=20;
    if (ticCount > 0) ticCount--;
    
    if (state > 0) {    //Auf/Abschwellen der LED-Helligkeit
        triangle+=dir;
        if (triangle >= 100) dir = -2;
        if (triangle <= 10)  dir = 2;
    } else triangle=80;
    showLeds(timeIndex, triangle);
    
    key = keypress();
      switch(state){
        case 0:  //Zeit einstellen (max. 5s
            if (key) {
                timer3s = 0;
                timeIndex++;
                if (timeIndex >= timeTableCount) timeIndex = 0;
            }
            timer3s ++;
            if (timer3s > 149) { //nach 3s
                ticCount = timeTable[timeIndex] * 50;
                state++;
            }
            break;
        case 1:  //erstes Absenken   
            if (key) {
                state = 90;
            }
            if (pos > POS_LOWER) {
              pos-=1;
              lift(pos);
            } else state++;
          break;
          
        case 2:  //Warte bis Minute rum oder fertig
            if (0 == (ticCount % 3000)) state ++;
            if (0 == ticCount) state = 90;
            if (key) {
                state = 90;
            }
            break;

        case 3:  //Hochziehen bis Mitte
          if (pos < POS_MIDDLE) {
            pos+=1;
            lift(pos);
          } else state ++;
            if (key) {
                state = 90;
            }
            break;
          
        case 4:  //langsam wieder Absenken
          if (pos > POS_LOWER) {
            pos-=0.5;
            lift(pos);
          } else state = 2;
            if (key) {
                state = 90;
            }
            break;
        
        case 90: //schnell Hochziehen 
          if (pos < POS_MIDDLE) {
            pos+=3;
            lift(pos);
          } else state ++;
          frq=3000;
          break;
          
        case 91:  //ganz langsam rausziehen
          if (pos < POS_UPPER) {
            pos+=0.1;
            lift(pos);
            tone(DO_SOUND, frq);
            frq+=6;
          } else state ++;
          break;

        case 92:  
          noTone(DO_SOUND);
          delay(2000);
          westminster_Sound();
          delay(1000);
          digitalWrite(DO_POWER, LOW); 
          delay(2000);
          break;
     }//switch state
  } //if millis
} //loop

